/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/

import jahuwaldt.js.ParamTools.AbstractParamMethod;
import jahuwaldt.js.ParamTools.ParamVerifier;
import jahuwaldt.js.param.Parameter;
import java.awt.Component;
import javax.measure.quantity.Angle;
import javax.measure.quantity.Dimensionless;
import javax.swing.JOptionPane;


/**
*  Calculates the Mach number based on cone half-angle and
*  the oblique conical shock angle.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: November 1, 2004
*  @version  December 20, 2023
**/
public class MachMethod extends AbstractParamMethod<Dimensionless> implements SharedData, ParamVerifier {
	
	/**
	*  Constructor for this method.
	*
	*  @param   thetaC  Label for the "cone half-angle" parameter.
	*  @param   thetaS  Label for the "shock angle" parameter.
	*  @param   gamma   Label fro the "ratio of specific heats" parameter.
	**/
	public MachMethod(String thetaC, String thetaS, String gamma) {
		_inputLabels = new String[3];
		_inputLabels[0] = thetaC;
		_inputLabels[1] = thetaS;
		_inputLabels[2] = gamma;
	}
	
	
	/**
	*  Calculates the Mach number using the conical shock model object
	*  found in the SharedData interface.
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=cone half-angle (thetaC), 1=oblique shock angle (thetaS)].
	*  @return  Mach number (M).
	**/
    @Override
	public Parameter<Dimensionless> calc(Parameter[] inputs) throws Exception {
		double thetaC = inputs[0].doubleValue(Angle.UNIT);
		double thetaS = inputs[1].doubleValue(Angle.UNIT);
		double gamma = inputs[2].getValue();
		
		conicalShock.setGamma(gamma);
		double M = conicalShock.freeStreamMach(thetaC, thetaS);
		
		return Parameter.valueOf(M, Dimensionless.UNIT);
	}

	/**
	*  Check to see if the input value is valid for this parameter.
	*
	*  @param   root   The root component containing the parameter input text field
	*                  for the parameter being verified.
	*  @param   value  The value of the parameter as input by the user.
	*  @return  If "value" is valid, it should be output.  If "value" is invalid or
	*           out of range, a valid value is output or Float.NaN if nothing else.
	*           If ParamInput (which calls this routine) recieves a value of Float.NaN,
	*           it will clear the parameter.
	**/
    @Override
	public Parameter check(Component root, Parameter value) {
		Parameter outValue = value;
		
		if (outValue.getValue() <= 1) {
			outValue = Parameter.valueOf(Double.NaN, Dimensionless.UNIT);
			
			JOptionPane.showMessageDialog ( root,
                                "Free-stream Mach number must be supersonic.",
								"Out of Range",
								JOptionPane.ERROR_MESSAGE );
		}
		
		return outValue;
	}

}
