/**
*  SupersonicCone  -- A Java application that calculates flow properties for a cone in supersonic flow.
*  Copyright (C) 2004-2023, Joseph A. Huwaldt
*
*  This program is free software; you can redistribute it and/or
*  modify it under the terms of the GNU General Public License
*  as published by the Free Software Foundation; either version 2
*  of the License, or (at your option) any later version.
*  
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/gpl.html
**/
import jahuwaldt.js.ParamTools.*;
import jahuwaldt.swing.*;
import jahuwaldt.util.GeneralFormat;
import java.awt.*;
import java.awt.desktop.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import static java.util.Objects.isNull;
import java.util.ResourceBundle;
import javax.measure.quantity.Angle;
import javax.measure.quantity.Dimensionless;
import javax.measure.unit.NonSI;
import javax.measure.unit.Unit;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;


/**
*  Application that allows the user to compute the properties related
*  to a cone moving through air at supersonic speeds.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author  Joseph A. Huwaldt   Date:  October 24, 2004
*  @version December 16, 2023
**/
public class SupersonicCone extends MDIApplication implements SharedData {
	private static final short	WIN_WIDTH = 680;				//	Application frame width & height.
	private static final short	WIN_HEIGHT = 550;
	
	//  A count of how many windows have been opened.
	private static int _windowCount = 0;
	
	/**
	*  The resource bundle for this application.
	*  The ResourceBundle below contains all of the strings used in this
	*  application.  ResourceBundles are useful for localizing applications.
	*  New localities can be added by adding additional properties files.
	**/
	private static final ResourceBundle RESBUNDLE = ResourceBundle.getBundle("appStrings", java.util.Locale.getDefault());
	

    /**
    *  Entry point for this application.  This is where it starts up from.
    **/
	public static void main( String[] args ) {
	    try {
			// Set the system look and feel to hide that hideous Java LAF.
			AppUtilities.setSystemLAF();

            // MacOS X users like the menu bar to be at the top of the screen.
            // This does nothing on non-MacOS platforms.
            System.setProperty("apple.laf.useScreenMenuBar", "true");

			// Create an instance of our application.
			new SupersonicCone(RESBUNDLE.getString("appName"));
		
		} catch (ParamException e) {
		    e.printStackTrace();
		    JOptionPane.showMessageDialog(null, e.getMessage(), RESBUNDLE.getString("paramErrorTitle"),
													JOptionPane.ERROR_MESSAGE );
			System.exit(0);
		    
	    } catch (MalformedURLException e) {
		    e.printStackTrace();
		    JOptionPane.showMessageDialog(null, e.getMessage(), RESBUNDLE.getString("paramDispErrorTitle"),
													JOptionPane.ERROR_MESSAGE );
			System.exit(0);
			
	    } catch (Throwable e) {
			e.printStackTrace();
			AppUtilities.showException(null, RESBUNDLE.getString("unexpectedTitle"),
							RESBUNDLE.getString("unexpectedMsg"), e);
			System.exit(0);
		}
	    
	}
	
	/**
	*  Constructor for our standard atmosphere application.
	*
	*  @param aName The name for the application.
	**/
	public SupersonicCone(String aName) throws Exception {
	    super(aName);
		
		//  Create the menu bar.
		JMenuBar menuBar = createMenuBar(false);
	    
		// Add the menu bar to this application.
		//  Cause the application to stay open when the last window is closed under MacOS.
		//  When the last window is closed, a menu bar will remain for the application with no
		//  window open.  This is standard MacOS behavior and will do nothing on other platforms.
		setFramelessJMenuBar( menuBar );
		
		//  Create a main window frame for this application.
		AppWindow window = new AppWindow(aName);
		addWindow(window);
		window.setVisible(true);
		
	}
	

	//-----------------------------------------------------------------------------------
	/**
	*  Handle the user choosing the "New..." from the File
	*  menu.  Creates and returns a reference to a new document window.
	**/
    @Override
	public Frame handleNew( ActionEvent event ) {
		Frame window = null;
		try {
		
			// Create an empty instance of this application's window.
			window = new AppWindow(RESBUNDLE.getString("appName"));
			addWindow(window);
			window.setVisible(true);

		} catch (ParamException e) {
			e.printStackTrace();
		    JOptionPane.showMessageDialog(null, e.getMessage(), RESBUNDLE.getString("paramErrorTitle"),
													JOptionPane.ERROR_MESSAGE );
		    
	    } catch (MalformedURLException e) {
		    e.printStackTrace();
		    JOptionPane.showMessageDialog(null, e.getMessage(), RESBUNDLE.getString("paramDispErrorTitle"),
													JOptionPane.ERROR_MESSAGE );
			
	    } catch (Exception e) {
			e.printStackTrace();
			AppUtilities.showException(null, RESBUNDLE.getString("unexpectedTitle"),
							RESBUNDLE.getString("unexpectedMsg"), e);
	    }

		return window;
	}


	/**
	*  Initializes and displays the menus associated with this application.
	**/
	private JMenuBar createMenuBar(boolean windowsMenu) throws NoSuchMethodException {

		//  Create a menu bar.
		JMenuBar menuBar = new JMenuBar();
	
		// Set up the file menu.
		List<String[]> menuStrings = new ArrayList();
        String[] row = new String[3];
		row[0] = RESBUNDLE.getString("newItemText");
		row[1] = RESBUNDLE.getString("newItemKey");
		row[2] = "handleNew";
		menuStrings.add(row);
        row = new String[3];
		row[0] = RESBUNDLE.getString("openItemText");
		row[1] = RESBUNDLE.getString("openItemKey");
		row[2] = null;
        menuStrings.add(row);
		menuStrings.add(new String[3]); //  Blank line
        row = new String[3];
		row[0] = RESBUNDLE.getString("closeItemText");
		row[1] = RESBUNDLE.getString("closeItemKey");
		row[2] = "handleClose";
		menuStrings.add(row);
        row = new String[3];
		row[0] = RESBUNDLE.getString("saveItemText");
		row[1] = RESBUNDLE.getString("saveItemKey");
		row[2] = null;
		menuStrings.add(row);
        row = new String[3];
		row[0] = RESBUNDLE.getString("saveAsItemText");
		row[1] = null;
		row[2] = null;
        menuStrings.add(row);
        
		JMenu menu = AppUtilities.buildMenu(this, RESBUNDLE.getString("fileMenuText"), menuStrings );
		menuBar.add( menu );
		
        //  Add a Quit menu item (if it isn't automatically present).
        if (!QuitJMenuItem.isAutomaticallyPresent()) {
            menu.addSeparator();
            menu.add(this.getQuitJMenuItem());
        }
        
        //  Add a Quit handler to ask the user if they want to save unsaved changes before quitting.
        setQuitHandler(new QuitHandler() {
            @Override
            public void handleQuitRequestWith​(QuitEvent e, QuitResponse response) {
                handleQuit(e, response);
            }
        });
        
		// Set up the edit menu.
		menuStrings.clear();
        row = new String[3];
		row[0] = RESBUNDLE.getString("undoItemText");
		row[1] = RESBUNDLE.getString("undoItemKey");
		row[2] = null;
		menuStrings.add(row);
        row = new String[3];
		row[0] = RESBUNDLE.getString("redoItemText");
		row[1] = null;
		row[2] = null;
		menuStrings.add(row);
		menuStrings.add(new String[3]); //  Blank line
        row = new String[3];
		row[0] = RESBUNDLE.getString("cutItemText");
		row[1] = RESBUNDLE.getString("cutItemKey");
		row[2] = null;
        menuStrings.add(row);
		row = new String[3];
		row[0] = RESBUNDLE.getString("copyItemText");
		row[1] = RESBUNDLE.getString("copyItemKey");
		row[2] = null;
        menuStrings.add(row);
		row = new String[3];
		row[0] = RESBUNDLE.getString("pasteItemText");
		row[1] = RESBUNDLE.getString("pasteItemKey");
		row[2] = null;
		menuStrings.add(row);
        
		menu = AppUtilities.buildMenu(this, RESBUNDLE.getString("editMenuText"), menuStrings );
		menuBar.add( menu );
		
		//  Create a Window's menu.
		if (windowsMenu) {
			menu = newWindowsMenu(RESBUNDLE.getString("windowsMenuText"));
			menuBar.add(menu);
		}
		
        //	Register an about menu item handler.
        setAboutHandler(createAboutHandler());
        
        //  Create an about menu item (if not automatically present).
        if (!AboutJMenuItem.isAutomaticallyPresent()) {
            //  Create Help menu.
            menu = new JMenu(RESBUNDLE.getString("helpMenuText"));
            menuBar.add(menu);
            
            //  Add the "About" item to the Help menu.
            menu.add(this.getAboutJMenuItem());
        }
        
		return menuBar;
	}

	/**
	*  Create an about handler for use in this application.
	**/
	public AboutHandler createAboutHandler() {
	
        AboutHandler about = new AboutHandler() {
            @Override
            public void handleAbout​(AboutEvent e) {
				
				//  Load in the application's icon image.
				Icon appIcon = null;
				
				try {
					URL imgURL = ClassLoader.getSystemResource(RESBUNDLE.getString("applicationIconURL") );
					appIcon = new ImageIcon(imgURL);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
				
				if (isNull(appIcon))
					//  Use a generic icon since this app can't read it's custom icon.
					appIcon = UIManager.getIcon("OptionPane.informationIcon");
				
				//  Read in the about box text information.
				String credits = readAboutText(RESBUNDLE.getString("aboutTextURLStr"));
				
				String aName = RESBUNDLE.getString("appName");
				String aVersion = RESBUNDLE.getString("appVersion");
				String aModDate = RESBUNDLE.getString("appModDate");
				String copyright = RESBUNDLE.getString("copyright");
				StandardMacAboutFrame f = new StandardMacAboutFrame(aName, aVersion + " - " + aModDate);
				f.setApplicationIcon(appIcon);
				f.setCopyright(copyright);
				f.setCredits(credits, "text/html");
				f.setHyperlinkListener(new HyperlinkListener() {
                    @Override
					public void hyperlinkUpdate(HyperlinkEvent e) {
						if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                            try {
                                MDIApplication.browse(e.getURL().toURI());
                            } catch (Exception err) {
                                err.printStackTrace();
                            }
						}
					}
				});
				f.setVisible(true);
			}
		};
		
		return about;
	}

	/**
	*  Method that handles reading in the contents of the text region of the about box from a text file.
	**/
	private String readAboutText(String urlStr) {
		String text;
		try {
		
			URL url = ClassLoader.getSystemResource(urlStr);
			
			InputStreamReader reader = new InputStreamReader( url.openStream(), "UTF-8" );
			StringWriter writer = new StringWriter();
			int character = reader.read();
			while (character != -1) {
				writer.write(character);
				character = reader.read();
			}
			text = writer.toString();
			
		} catch (IOException e) {
			e.printStackTrace();
			
			//  Create a standard set of credits if we didn't find the text file.
			text = "<html><body><b>Author:</b><br>Joseph A. Huwaldt<br>";
			text += "<a href=\"mailto:jhuwaldt@mac.com\">jhuwaldt@mac.com</a><br>";
			text += "<P ALIGN=CENTER><BR>" + getResourceBundle().getString("appName") + " comes with ABSOLUTELY NO WARRANTY;";
			text += "</body></html>";
		}
		
		return text;
	}


	//-----------------------------------------------------------------------------------
	/**
	*  The main window frame for this application.
	**/
	private class AppWindow extends JFrame {
	
		//  A database to hold our parameters.
		private final ParamDatabase _pdb = new ParamDatabase();
	
		
		/**
		*  Construct a new application window instance.
		*
		*  @param aName  The name of this application.
		**/
        @SuppressWarnings({"LeakingThisInConstructor", "OverridableMethodCallInConstructor"})
		public AppWindow(String aName) throws Exception {
            super(aName + (++_windowCount > 1 ? " " + String.valueOf(_windowCount) : ""));
		
			//  Have the window dispose of itself when it closes.
			this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		
			//  Create a menu bar for this window.
			JMenuBar menuBar = createMenuBar(true);
			setJMenuBar(menuBar);
	    
			//	Create parameter objects for each of this program's parameters.
			initParams();
			
			//	Layout the content of the frame.
			Container cp = this.getContentPane();
			cp.setLayout(new BorderLayout());
			
			// Create an info region at the top.
			cp.add(layoutInfoRgn(aName, RESBUNDLE.getString("versionTxt") + " " + RESBUNDLE.getString("appVersion"),
					RESBUNDLE.getString("writtenByTxt") + "  " + RESBUNDLE.getString("appAuthor"),
					RESBUNDLE.getString("appModDate")), BorderLayout.NORTH);

			JPanel centerPanel = new JPanel(new BorderLayout());
			cp.add(centerPanel, BorderLayout.CENTER);
			
			//  Load the illustration image into the center of the window.
			URL imgURL = SupersonicCone.class.getResource(RESBUNDLE.getString("illustrationLabelImg"));
			ImageIcon icon = new ImageIcon(imgURL);
			centerPanel.add(new JLabel(icon), BorderLayout.CENTER);
			
			//	Display input boxes for all the parameters in this window.
			//	Create a list of parameters for the top parameter panel.
			List<String> pLabels = new ArrayList();
			pLabels.add("Mach");
			pLabels.add("thetaC");
			pLabels.add("thetaS");
			pLabels.add("gamma");
			
			//	Add the input parameters to be displayed to the window.
			centerPanel.add(layoutOrderedParams(_pdb, pLabels, true), BorderLayout.NORTH);
			
			//	Create a list of parameters for the bottom parameter panel.
			pLabels.clear();
			pLabels.add("TT/T");
			pLabels.add("PT/P");
			pLabels.add("rhoT/rho");
			pLabels.add("Tc/T0");
			pLabels.add("Pc/P0");
			pLabels.add("rhoc/rho0");
			pLabels.add("Mc");

			//	Add the output parameters to be displayed to the window.
			centerPanel.add(layoutOrderedParams(_pdb, pLabels, false), BorderLayout.SOUTH);

			//  Size the window and then display it.
			AppUtilities.positionWindow( this, WIN_WIDTH, WIN_HEIGHT );
			
		}
		

		/**
		*  Sets the title for this frame to the specified string.  Also notifies the main
		*  application class so that the "Windows" menu gets updated too.
		*
		*  @param  title The title to be displayed in the frame's border.
		*                A null value is treated as an empty string, "".
		**/
        @Override
		public void setTitle(String title) {
			super.setTitle(title);
			MDIApplication.windowTitleChanged(this);
		}
		
		/**
		*  Creates and lays out a panel that contains the header information
		*  and "clear all" button.
		*
		*  @param  label1  The primary message to display in the info region.
		*  @param  label2  The secondary message to display in the info region.
		*  @return A panel containing the header information and clear all button.
		**/
		private JPanel layoutInfoRgn(String label1, String label2, String label3, String label4) {
			
			JPanel infoRgnPanel = new JPanel(new BorderLayout());
			
			// Place information labels.
			Box infoPanel1 = new Box(BoxLayout.Y_AXIS);
			JLabel infoLabel = new JLabel(label1);
			infoPanel1.add(infoLabel);
			infoPanel1.add(Box.createVerticalStrut(5));
			infoLabel = new JLabel(label2);
			infoPanel1.add(infoLabel);

			Box infoPanel2 = new Box(BoxLayout.Y_AXIS);
			infoLabel = new JLabel(label3);
			infoPanel2.add(infoLabel);
			infoPanel2.add(Box.createVerticalStrut(5));
			infoLabel = new JLabel(label4);
			infoPanel2.add(infoLabel);

			Box infoPanel3 = new Box(BoxLayout.X_AXIS);
			infoPanel3.add(infoPanel1);
			infoPanel3.add(Box.createHorizontalStrut(40));
			infoPanel3.add(infoPanel2);

			infoRgnPanel.add(infoPanel3, BorderLayout.WEST);
			
			//	Place Clear button.
			JButton clearAllButton = new JButton(RESBUNDLE.getString("clearAllBtnStr"));
			clearAllButton.addActionListener(new ActionListener() {
                    @Override
					public void actionPerformed( ActionEvent evt ) {
						AutoParam.clearAllParams(_pdb);
					}
				});
			clearAllButton.setToolTipText(RESBUNDLE.getString("clearAllToolTip"));
			infoRgnPanel.add(clearAllButton, BorderLayout.EAST);
			
			infoRgnPanel.setBorder(new EmptyBorder(10,20,10,20));
			
			return infoRgnPanel;
		}

		/**
		*  Lays out the selected parameters in the parameter database by
		*  adding them as components in a grid bag layout to a new panel
		*  and returning that panel.
		*
		*  @param  pdb      The parameter database being used by this
		*                   program.
		*  @param  pLabels  An list of string labels for the parameters that are
		*                   to be displayed.
		*  @param  enabled  Set to true to enable the text edit boxes for editing, false to disable them.
		*  @return A panel containing all the parameter input boxes.
		**/
		private JPanel layoutOrderedParams(ParamDatabase pdb, List<String> pLabels, boolean enabled) throws MalformedURLException {
			int columns = 3;
			int xPos=0, yPos=0;
			
			//	Set up for grid layout.
			GridBagConstraints constraints = new GridBagConstraints();
			GridBagLayout gridbag = new GridBagLayout();
			JPanel panel = new JPanel(gridbag);
			
			//	Loop over elements in database, placing each one in this container.
			for (String label : pLabels) {
				AppUtilities.buildConstraints(constraints, xPos,yPos, 1,1, 100/columns, 10);
				constraints.fill = GridBagConstraints.BOTH;		//	How the component should fill cell.
				constraints.anchor = GridBagConstraints.CENTER;	//	Justification in cell
				
				//	Get the next element from database.
				AutoParam theParam = (AutoParam)pdb.get(label);
				
				if (theParam == null)
					throw new NullPointerException(RESBUNDLE.getString("paramNotFoundMsg1") + " " + label
								+ " " + RESBUNDLE.getString("paramNotFoundMsg2"));
					
				//	Create text field for inputing this parameter's value.
				AutoParamInput thePI = new AutoParamInput( theParam );
				thePI.setEditable(enabled);
				gridbag.setConstraints(thePI, constraints);
				panel.add(thePI);
				
				++xPos;
				if (xPos >= columns) {
					xPos = 0;
					++yPos;
				}
			}
			
			return panel;
		}
			

		/**
		*  Initialize all the parameter objects used by this program.
		**/
		public void initParams() throws Exception {
			
		//  Create lists of units according to type.
			Unit DEG = NonSI.DEGREE_ANGLE;
			Unit[] angularUnits = {	DEG, Angle.UNIT };
			
		//	Create an object to contain the inputs we are using.
			AutoParamInputs inputs = new AutoParamInputs();
			inputs.theDatabase = _pdb;
			inputs.numFormat = new GeneralFormat();
			
		//	Gamma number parameter
			inputs.label = "gamma";
			inputs.desc = "Ratio of specific heats.";
			inputs.symbolURL = getResource("GammaLabelImg");
			inputs.theUnits = Dimensionless.UNIT;
			inputs.availUnits = null;
			inputs.theMethod = null;
			inputs.verifierMethod = new GammaVerifier();
			AutoParam aParam = new AutoParam( inputs );
			aParam.setValue(1.4);
			
		//	Mach number parameter
			inputs.label = "Mach";
			inputs.desc = "Free-stream Mach number.";
			inputs.symbolURL = getResource("MinfLabelImg");
			inputs.theUnits = Dimensionless.UNIT;
			inputs.availUnits = null;
			inputs.theMethod = new MachMethod("thetaC", "thetaS", "gamma");
			inputs.verifierMethod = (ParamVerifier)inputs.theMethod;
			new AutoParam( inputs );
			
		//  Cone half-angle paramter.
			inputs.label = "thetaC";
			inputs.desc = "Cone half-angle.";
			inputs.symbolURL = getResource("thetaCLabelImg");
			inputs.theUnits = DEG;
			inputs.availUnits = angularUnits;
			inputs.theMethod = new ThetaCMethod("Mach", "thetaS", "gamma");
			inputs.verifierMethod = (ParamVerifier)inputs.theMethod;
			new AutoParam( inputs );
			
		//  Shock angle paramter.
			inputs.label = "thetaS";
			inputs.desc = "Oblique shock wave angle.";
			inputs.symbolURL = getResource("thetaSLabelImg");
			inputs.theUnits = DEG;
			inputs.availUnits = angularUnits;
			inputs.theMethod = new ThetaSMethod("Mach", "thetaC", "gamma");
			inputs.verifierMethod = (ParamVerifier)inputs.theMethod;
			new AutoParam( inputs );
			
		//  Total temperature ratio paramter.
			inputs.label = "TT/T";
			inputs.desc = "Total-temp. to static temp. ratio at cone surface.";
			inputs.symbolURL = getResource("TT_TLabelImg");
			inputs.theUnits = Dimensionless.UNIT;
			inputs.availUnits = null;
			inputs.theMethod = new TT_TMethod("Mach", "thetaS", "gamma");
			inputs.verifierMethod = null;
			new AutoParam( inputs );
			
		//  Total pressure ratio paramter.
			inputs.label = "PT/P";
			inputs.desc = "Total-pressure to static pressure ratio at cone surface.";
			inputs.symbolURL = getResource("PT_PLabelImg");
			inputs.theUnits = Dimensionless.UNIT;
			inputs.availUnits = null;
			inputs.theMethod = new PT_PMethod("Mach", "thetaS", "gamma");
			inputs.verifierMethod = null;
			new AutoParam( inputs );
			
		//  Total density ratio paramter.
			inputs.label = "rhoT/rho";
			inputs.desc = "Total-density to static density ratio at cone surface.";
			inputs.symbolURL = getResource("rhoT_rhoLabelImg");
			inputs.theUnits = Dimensionless.UNIT;
			inputs.availUnits = null;
			inputs.theMethod = new RhoT_RhoMethod("Mach", "thetaS", "gamma");
			inputs.verifierMethod = null;
			new AutoParam( inputs );
			
		//  Ratio of static temperature at the cone surface to the free-
		//  stream static temperature paramter.
			inputs.label = "Tc/T0";
			inputs.desc = "Static temp. at cone surface to free-stream static temp. ratio.";
			inputs.symbolURL = getResource("Tc_TinfLabelImg");
			inputs.theUnits = Dimensionless.UNIT;
			inputs.availUnits = null;
			inputs.theMethod = new Tc_T0Method("Mach", "thetaS", "gamma");
			inputs.verifierMethod = null;
			new AutoParam( inputs );
			
		//  Ratio of static pressure at the cone surface to the free-
		//  stream static pressure paramter.
			inputs.label = "Pc/P0";
			inputs.desc = "Static pressure at cone surface to free-stream static pressure ratio.";
			inputs.symbolURL = getResource("Pc_PinfLabelImg");
			inputs.theUnits = Dimensionless.UNIT;
			inputs.availUnits = null;
			inputs.theMethod = new Pc_P0Method("Mach", "thetaS", "gamma");
			inputs.verifierMethod = null;
			new AutoParam( inputs );
			
		//  Ratio of static density at the cone surface to the free-
		//  stream static density paramter.
			inputs.label = "rhoc/rho0";
			inputs.desc = "Static density at cone surface to free-stream static density ratio.";
			inputs.symbolURL = getResource("rhoC_rhoinfLabelImg");
			inputs.theUnits = Dimensionless.UNIT;
			inputs.availUnits = null;
			inputs.theMethod = new Rhoc_Rho0Method("Mach", "thetaS", "gamma");
			inputs.verifierMethod = null;
			new AutoParam( inputs );
			
		//  Mach number at the surface of the cone.
			inputs.label = "Mc";
			inputs.desc = "Mach number at the cone surface.";
			inputs.symbolURL = getResource("McLabelImg");
			inputs.theUnits = Dimensionless.UNIT;
			inputs.availUnits = null;
			inputs.theMethod = new McMethod("Mach", "thetaS", "gamma");
			inputs.verifierMethod = null;
			new AutoParam( inputs );
			
		//	Have each parameter link up with the parameters that it depends on.
			AutoParam.linkAllParams(_pdb);
		}
		
		/**
		*  Method that uses the Java class loader to return the URL to a resource with the
		*  specified resource bundle item name.
		**/
		private URL getResource(String resName) {
			return this.getClass().getResource(RESBUNDLE.getString(resName));
		}
		
	}   //  end class AppWindow
	
}   //  end class SupersonicCone
