/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/

import jahuwaldt.js.ParamTools.AbstractParamMethod;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Angle;
import javax.measure.quantity.Dimensionless;


/**
*  Calculates static temperature ratio (cone surface to free-stream)
*  from the free stream Mach number and conical shock angle.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: November 2, 2004
*  @version  December 20, 2023
**/
public class Tc_T0Method extends AbstractParamMethod<Dimensionless> implements SharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   Mach    Label for the "free-stream Mach number" parameter.
	*  @param   thetaS  Label for the "shock angle" parameter.
	*  @param   gamma   Label fro the "ratio of specific heats" parameter.
	**/
	public Tc_T0Method(String Mach, String thetaS, String gamma) {
		_inputLabels = new String[3];
		_inputLabels[0] = Mach;
		_inputLabels[1] = thetaS;
		_inputLabels[2] = gamma;
	}
	
	
	/**
	*  Calculates static temperature ratio (cone surface to free-stream)
	*  from the free stream Mach number and conical shock angle.
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=free-stream Mach number (Mach), 1=oblique shock angle (thetaS)].
	*  @return  Static temperature ratio (cone surface to free-stream -- Tc/T0).
	**/
    @Override
	public Parameter<Dimensionless> calc(Parameter[] inputs) throws Exception {
		double Mach = inputs[0].doubleValue(Dimensionless.UNIT);
		double thetaS = inputs[1].doubleValue(Angle.UNIT);
		double gamma = inputs[2].getValue();
		
		conicalShock.setGamma(gamma);
		double result = conicalShock.coneT_T0(Mach, thetaS);
		
		return Parameter.valueOf(result, Dimensionless.UNIT);
	}

}
