/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/

import jahuwaldt.aero.ConicalShock;
import jahuwaldt.js.ParamTools.AbstractParamMethod;
import jahuwaldt.js.ParamTools.ParamVerifier;
import jahuwaldt.js.param.Parameter;
import java.awt.Component;
import javax.measure.quantity.Angle;
import javax.measure.quantity.Dimensionless;
import javax.swing.JOptionPane;


/**
*  Calculates the oblique conical shock angle based on free-stream
*  Mach number and the cone half-angle.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: November 1, 2004
*  @version  December 20, 2023
**/
public class ThetaSMethod extends AbstractParamMethod<Angle> implements SharedData, ParamVerifier {
	
	/**
	*  Constructor for this method.
	*
	*  @param   Mach  Label for the "free-stream Mach number" parameter.
	*  @param   thetaC  Label for the "cone half-angle" parameter.
	*  @param   gamma   Label fro the "ratio of specific heats" parameter.
	**/
	public ThetaSMethod(String Mach, String thetaC, String gamma) {
		_inputLabels = new String[3];
		_inputLabels[0] = Mach;
		_inputLabels[1] = thetaC;
		_inputLabels[2] = gamma;
	}
	
	
	/**
	*  Calculates the oblique conical shock angle based on the free-stream
	*  Mach number and the cone half-angle.
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=free-stream Mach number (Mach), 1=cone half-angle (thetaC)].
	*  @return  Oblique conical shock angle.
	**/
    @Override
	public Parameter<Angle> calc(Parameter[] inputs) throws Exception {
		double Mach = inputs[0].doubleValue(Dimensionless.UNIT);
		double thetaC = inputs[1].doubleValue(Angle.UNIT);
		double gamma = inputs[2].getValue();
		
		conicalShock.setGamma(gamma);
		double thetaS = conicalShock.shockAngle(Mach, thetaC, ConicalShock.WEAK);
		
		return Parameter.valueOf(thetaS, Angle.UNIT);
	}

	/**
	*  Check to see if the input value is valid for this parameter.
	*
	*  @param   root   The root component containing the parameter input text field
	*                  for the parameter being verified.
	*  @param   value  The value of the parameter as input by the user.
	*  @return  If "value" is valid, it should be output.  If "value" is invalid or
	*           out of range, a valid value is output or Float.NaN if nothing else.
	*           If ParamInput (which calls this routine) recieves a value of Float.NaN,
	*           it will clear the parameter.
	**/
    @Override
	public Parameter check(Component root, Parameter value) {
		Parameter outValue = value;
		
		if (outValue.isGreaterThan(Parameter.HALFPI_ANGLE)) {
			outValue = Parameter.HALFPI_ANGLE;
			
			JOptionPane.showMessageDialog ( root,
                                "The shock angle can not be greater than 90 degrees.",
								"Out of Range",
								JOptionPane.ERROR_MESSAGE );
		}
		
		return outValue;
	}

}
