/*
*   AdvParam - Advanced Param object that includes some additional information.
*
*   Copyright (C) 1998-2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.ParamTools;

import java.net.URL;
import javax.measure.unit.Unit;
import javax.measure.quantity.Quantity;

import jahuwaldt.util.HelpSystem;
import jahuwaldt.js.param.Parameter;


/**
*  <p>  An advanced parameter type that includes additional
*       information such as a link to a help web page and
*       an optional graphical symbol rather than a text label.
*  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: July 12, 1999
*  @version May 27, 2012
**/
public class AdvParam<Q extends Quantity> extends Param<Q> {

	private static final long serialVersionUID = -3013881648224017580L;

	/**
	*  An object containing a method for verifying the
	*  validity of the value input to this parameter.
	**/
	private ParamVerifier _verifier = null;

	/**
	*  URL to the image file to use as a symbol in
	*  place of the text label when displaying
	*  this parameter (like in a ParamInput panel).
	**/
	private URL _symbolURL = null;

	
	/**
	*  URL to an image file to use for the
	*  "help/info" button when displaying this parameter
	*  in an AvdParamInput panel.
	**/
	private URL _helpIconURL = null;

	/**
	*  Reference to help information for this parameter.
	**/
	private HelpSystem _helpSys = null;

	/**
	*  String describing this parameter.
	**/
	private String _desc = null;
	
	
	//-----------------------------------------------------------------------------
	/**
	*  Constructs an advanced parameter object using a
	*  label string and a <code>Parameter</code> value.
	*
	*  @param   label  The label to be attached to this object.
	*  @param   value  The value for the parameter to hold.
	**/
	public AdvParam( String label, Parameter<Q> value ) {
		super( label, value );
	}

	/**
	*  Constructs an advanced parameter object using a
	*  label string and the units for the parameter while
	*  setting the parameter's initial value to <code>Double.NaN</code>
	*  (undefined).
	*
	*  @param  label    The label to be attached to this object.
	*  @param  theUnits The units associated with this parameter.
	**/
	public AdvParam( String label, Unit<Q> theUnits ) {
		super( label, theUnits );
	}

	/**
	*  Constructs an advanced parameter object using a
	*  label string, <code>double</code> value, and units for the parameter.
	*
	*  @param  label    The label to be attached to this object.
	*  @param  value    The value for the parameter to hold.
	*  @param  theUnits The units associated with this parameter.
	**/
	public AdvParam( String label, double value, Unit<Q> theUnits ) {
		super( label, value, theUnits );
	}

	/**
	*  Constructs an advanced parameter object that takes a
	*  label string, label symbol image URL, and the units and
	*  sets parameter to an initial value of <code>Double.NaN</code> (undefined).
	*
	*  @param  label       The label to be attached to this object.
	*  @param  theUnits    The units associated with this parameter.
	*  @param  symbolURL   The URL to an image file to use in place of the text
	*                      label string.
	**/
	public AdvParam( String label, URL symbolURL, Unit<Q> theUnits ) {
		super( label, theUnits );
		_symbolURL = symbolURL;
	}

	/**
	*  Constructs an advanced parameter object that takes a
	*  label string, label symbol image URL, <code>double</code> value,
	*  and units for the parameter.
	*
	*  @param  label       The label to be attached to this object.
	*  @param  symbolURL   The URL to an image file to use in place of the text
	*                      label string.
	*  @param  value       The value for the parameter to hold.
	*  @param  theUnits    The units associated with this parameter.
	**/
	public AdvParam( String label, URL symbolURL, double value, Unit<Q> theUnits ) {
		super( label, value, theUnits );
		_symbolURL = symbolURL;
	}

	/**
	*  Constructs an advanced parameter object that takes a
	*  label string, label symbol image URL, <code>double</code> value,
	*  and units for the parameter.
	*
	*  @param  label       The label to be attached to this object.
	*  @param  symbolURL   The URL to an image file to use in place of the text
	*                      label string.
	*  @param  value       The value for the parameter to hold.
	**/
	public AdvParam( String label, URL symbolURL, Parameter<Q> value ) {
		super( label, value );
		_symbolURL = symbolURL;
	}


	//-----------------------------------------------------------------------------
	/**
	*  Method to return the URL to the symbol image file that can be
	*  used in place of the string label of this parameter.
	*
	*  @return Returns the URL to the parameter symbol image file.
	**/
	public final URL getSymbolURL() {
		return _symbolURL;
	}

	/**
	*  Set help information to this parameter.  Help information
	*  is used by display/input containers to provide the user
	*  access to user assistance information.
	*
	*  @param  buttonImgURL  URL to an image file containing the
	*                        image to be used with the help button.
	*  @param  help          Reference to a class that provides user assistance
	*                        information for this paramter.
	**/
	public void setHelp( URL buttonImgURL, HelpSystem help ) {
		_helpIconURL = buttonImgURL;
		_helpSys = help;
	}

	/**
	*  Removes reference to help information used by this parameter.
	**/
	public void removeHelp() {
		_helpIconURL = null;
		_helpSys = null;
	}

	/**
	*  Returns true if there is help information available, false if
	*  not.
	*
	*  @return  Returns true if help is avialable, false if it is not.
	**/
	public boolean isHelpAvail() {
		return (_helpSys != null && _helpSys.helpAvailable());
	}

	/**
	*  Return's a reference to the help information associated with
	*  this paramter.  If there is none available, null is returned.
	*
	*  @return  Returns a reference to the help information for this
	*           parameter.
	**/
	public HelpSystem getHelpSystem() {
		return _helpSys;
	}

	/**
	*  Method that returns the URL to the image file that can be
	*  used for the "help/info" button displayed in an
	*  {@link AdvParamInput} panel.
	*
	*  @return Returns the URL to the parameter help/info button image file.
	*          Returns null if no image is available.
	**/
	public URL getHelpImageURL() {
		return _helpIconURL;
	}

	/**
	*  Set the ParamVerifier object (and method) to be used to
	*  verify the validity of values entered for this parameter.
	*
	*  @param  verifierMethod   The ParamVerifier object to use to
	*                           verify the validity of this parameter.
	**/
	public void setParamVerifier( ParamVerifier verifierMethod ) {
		_verifier = verifierMethod;
	}

	/**
	*  Returns a reference to the ParamVerifier associated with this
	*  parameter object.
	*
	*  @return Returns a reference to the ParamVerifier method
	*          associated with this parameter.
	**/
	public final ParamVerifier getVerifier() {
		return _verifier;
	}

	/**
	*  Set the description of this parameter.
	**/
	public void setDescription(String description) {
		_desc = description;
	}
	
	/**
	*  Return the description of this parameter.  Null is returned
	*  if there is no description.
	**/
	public String getDescription() {
		return _desc;
	}
	
	
}


