/*******************************************************************************

	File:		PreferencesJMenuItem.java
	Author:		Steve Roy <steve@sillybit.com>
				
	Part of MRJ Adapter, a unified API for easy integration of Mac OS specific
	functionality within your cross-platform Java application.
	
	This library is open source and can be modified and/or distributed under
	the terms of the Artistic License.
	<http://mrjadapter.dev.java.net/license.html>
	
	Change History:
	01/31/03	Created this file - Steve

*******************************************************************************/

package jahuwaldt.swing;

import javax.swing.JMenuItem;

/**
 * This is the Swing implementation of the Preferences menu item.
 * <p>
 * On Mac OS X, this menu item is always automatically included in the menu
 * bar of the application. On other platforms, it never is. You can find out
 * at runtime if the menu item is automatically included with the
 * <code>isAutomaticallyPresent()</code> method and then add it yourself if
 * it isn't. This will make your code cross-platform while letting the
 * application do the right thing for the current platform.
 * <p>
 * In the case where the Preferences menu item is automatically included,
 * this menu item is really just a placeholder for the actual native menu
 * item, passing off operations to and from the native menu item where
 * possible. Of course, when this is the case, not all methods of this class
 * will be functional. However, there is no harm in calling dysfunctional
 * methods, other than your user interface not matching your requests.
 * <p>
 * The methods that work on all platforms are the following.
 * <ul>
 * <li>addActionListener</li>
 * <li>removeActionListener</li>
 * <li>setAction (only making the action the listener will actually work on
 * all platforms)</li>
 * <li>setEnabled()</li>
 * <li>isEnabled()</li>
 * </ul>
 * 
 * Based pm MRJ Adapter 1.2, Modified by: Joseph A. Huwaldt
 * @version December 16, 2023
 */
public class PreferencesJMenuItem extends JMenuItem {

    /**
     * Construct a Preferences menu item. This method is package private so only the
     * <code>Application</code> class can create a Preferences menu item.
     */
    PreferencesJMenuItem() {
        super("Preferences");
    }

    /**
     * Get whether this menu item is automatically present in the menu bar of the current
     * underlying platform.
     *
     * @return whether this menu item is automatically present
     */
    public static boolean isAutomaticallyPresent() {
        return MacOSUtilities.isMacOS();
    }
}
